import { Component, Input, Optional, OnInit } from '@angular/core';
import { FormGroup, FormGroupName, ControlContainer } from '@angular/forms';
import { KeysPipe } from '@pipes/keys.pipe';

@Component({
	selector: 'app-dynaform',
	templateUrl: './dynaform.component.html',
	styleUrls: ['./dynaform.component.scss'],
	providers: [ KeysPipe ]
})
export class DynaformComponent implements OnInit {

	/*
	 * USAGE
	 * Supply with either a FormGroup or the (string) name of a FormGroup, and the form meta data
	 * e.g.
	 * <app-dynaform [formGroup]="myForm" [meta]="myFormMetaData"></app-dynaform>
	 * <app-dynaform formGroupName="myFormGroupName" [meta]="myFormMetaData"></app-dynaform>
	 *
	 * If supplied with just a FormGroupName it will retieve the FormGroup from the injected ControlContainer
	 */

	@Input()
	formGroup?: FormGroup;

	@Input()
	formGroupName?: FormGroupName;

	@Input()
	set meta(data) {
		this.formMetaData = data;
	}

	formMetaData: any; // TODO: Tighten up type
	controlNames: Array<string>;

	constructor(
		@Optional() private cc: ControlContainer
	) {}

	ngOnInit() {
		console.log('Dynaform Component');
		if (this.formGroup) {
			console.log('Initailised with FormGroup dirtectly: Controls >');
			console.log(Object.keys(this.formGroup.controls));
		}
		if (this.formGroupName) {
			console.log('Initailised with FormGroupName:', this.formGroupName);
			this.formGroup = <FormGroup>this.cc.control; // Get the associated formGroup from the ControlContainer
			console.log('Loading Controls from ControlContainer >');
			console.log(Object.keys(this.formGroup.controls));
		}
		if (!this.formGroup) {
			throw new Error('Dynaform Component Initialised without [formGroup] or formGroupName');
		}

	}

}
